//
// intel Native Signal Processing Library demo program
//
// Author:	Robert Rossmair
//		Wagelwitz, Rodaer Str. 6
//		04688 Mutzschen
//		Germany
//
// e-mail:	Robert.Rossmair@t-online.de
//
// Version:	21-May-1999
//
//
// This is a free translation of
// <ISPL installation directory>\examples\firdemo\firmaker.cpp

unit FIRMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ZoomDisp, StdCtrls, ComCtrls, ExtCtrls,
{$IFDEF NoMath} // Math unit not included in Delphi 3 Standard
  RRMath,
{$ELSE}
  Math,
{$ENDIF}
  NSP, NumEdit, NSPInfo, Buttons, SplitBox;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    TapsBar: TTrackBar;
    FpBar: TTrackBar;
    FsBar: TTrackBar;
    FilterList: TComboBox;
    WindowList: TComboBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    EditFp: TFloatEdit;
    EditFs: TFloatEdit;
    EditTaps: TIntEdit;
    SignalDisplayType: TComboBox;
    Label6: TLabel;
    PosX: TLabel;
    PosY: TLabel;
    SpeedButton1: TSpeedButton;
    SplitBox1: TSplitBox;
    SignalWin: TZoomDisplay;
    FilterWin: TZoomDisplay;
    BetaBar: TTrackBar;
    EditBeta: TFloatEdit;
    Label7: TLabel;
    procedure TapsBarChange(Sender: TObject);
    procedure FpBarChange(Sender: TObject);
    procedure FsBarChange(Sender: TObject);
    procedure EditTapsChange(Sender: TObject);
    procedure EditFpChange(Sender: TObject);
    procedure EditFsChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SignalWinPaint(Sender: TObject);
    procedure ListChange(Sender: TObject);
    procedure FilterWinPaint(Sender: TObject);
    procedure SignalWinWorldPosChange(Sender: TObject; X, Y: Double);
    procedure FilterWinWorldPosChange(Sender: TObject; X, Y: Double);
    procedure SignalDisplayTypeChange(Sender: TObject);
    procedure ShowInfoDlg(Sender: TObject);
    procedure BetaBarChange(Sender: TObject);
    procedure EditBetaChange(Sender: TObject);
    procedure SignalWinGetVertScalePoint(Sender: TObject;
      var Value: Extended);
    procedure FilterWinGetVertScalePoint(Sender: TObject;
      var Value: Extended);
    procedure SignalWinMouseLeave(Sender: TObject);
  private
    Ysmin, Ysmax,
    Yfmin, Yfmax: Single;
    procedure Generate;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

const
  // filter types
  ftLowPass	= 0;
  ftHighPass	= 1;
  ftBandPass	= 2;

  // windowing functions
  wfRectangular	= 0;
  wfBartlett	= 1;
  wfHann	= 2;
  wfHamming	= 3;
  wfBlackmanStd	= 4;
  wfBlackmanOpt	= 5;
  wfKaiser      = 6;

  FirLen	= 128;	// Max number of filter taps
  BufLen	= 256;	// Number of samples of input signal
  Order		= 8;	// Order of fast Fourier transform
  XMARG		= 120;
  NOFTONE	= 6;
  STEPFREQ	= 0.05;
var
  InpBuf: array[0..BufLen] of Single;
  OutBuf: array[0..BufLen] of Single;
  FirBuf: array[0..FirLen-1] of Single;
  // DlyBuf: array[0..FirLen-1] of Single;
  Length: Integer;

{$IFDEF NoMath}
const
  MaxSingle = 3.4e+38;

function Log10(X: Extended): Extended;
asm
	FLDLG2
	FLD     X
	FYL2X
	FWAIT
end;
{$ENDIF}

procedure TForm1.Generate;
var
  P1, P2: Integer;
  i, j, Center: Integer;
  a, b,
  Ymin,
  Ymax,
  Freq1,
  Freq2: Single;
  State: TNSPFirState;
  ToneSt: array[0..NOFTONE-1] of TNSPSToneState;
begin
  if csLoading in ComponentState then Exit;
  P1 := FpBar.Position;
  P2 := FsBar.Position;
  if FilterList.ItemIndex = ftBandPass then
  if P2 <= P1 then
  begin
    if P1 < FpBar.Max then
    begin
      FsBar.Position := FpBar.Position + 1;
      FsBarChange(Self);
    end else
    begin
      FpBar.Position := FsBar.Position - 1;
      FpBarChange(Self);
    end;
    Exit;
  end;

   /// Get current values of active variables

  Length := EditTaps.Value;
  Freq1 := EditFp.Value;
  Freq2 := EditFs.Value;

   /// Prepare signals mixture

   ////////////////////////////////////////// NSP. Tone
   ////////////////////////////////////////// NSP. Tone

   // Tone generator state structure array

   for j := 0 to BufLen-1 do InpBuf[j] := 0.0;
   for i := 0 to NOFTONE-1 do
   begin
      // Initialize tone generator with freq, phase, magnitude
      nspsToneInit(0.03+STEPFREQ*i, 0.0, 1.0, ToneSt[i]);
      // Generate tone signal in buffer of given length
      nspsbTone(ToneSt[i], @OutBuf, BufLen);
      for j := 0 to BufLen-1 do InpBuf[j] := InpBuf[j] + OutBuf[j];
   end;

   // Warning. Even number of coefficients
   if not Odd(Length) then Inc(Length);
   Center := Length div 2;

   /// Calculate "raw" values of the filter coefficients

   // Handbook for Digital Signal Processing. ed. S.K.Mitra, J,F.Kaiser
   // John Wiley & Sons. 1993. p.176.

  case FilterList.ItemIndex of
    ftLowPass:
      begin   // Lowpass filter
	FirBuf[Center] := Freq1;
	for i := 1 to Center do
	begin
	  FirBuf[Center-i] := sin(PI*Freq1*i)/(PI*i);
	  FirBuf[Center+i] := FirBuf[Center-i];
	end;
      end;
   ftHighPass:	// Highpass filter
      begin
	FirBuf[Center] := 1.0-Freq1;
	for i := 1 to Center do
	begin
	  FirBuf[Center-i] := -sin(PI*Freq1*i)/(PI*i);
	  FirBuf[Center+i] := FirBuf[Center-i];
	end;
      end;
   ftBandPass:	// Bandpass filter
      begin
	FirBuf[Center] := Freq2-Freq1;
	for i := 1 to Center do
	begin
	  FirBuf[Center-i] := (sin(PI*freq2*i)-sin(PI*freq1*i))/(PI*i);
	  FirBuf[Center+i] := FirBuf[Center-i];
	end;
      end
   end;

   ////////////////////////////////////////// NSP. Windowing
   ////////////////////////////////////////// NSP. Windowing

  case WindowList.ItemIndex of
    wfRectangular: ;
      // Use Rectangular windowing function
      // (that is, in this case -buffer already zeroed
      // outside the window-, no windowing at all)
    wfBartlett:
      // Use Bartlett windowing function
      nspsWinBartlett(@FirBuf,Length);
    wfHann:
      nspsWinHann(@FirBuf,Length);
    wfHamming:
      // Use Hamming windowing function
      nspsWinHamming(@FirBuf,Length);
    wfBlackmanStd:
      nspsWinBlackmanStd(@FirBuf,Length);
    wfBlackmanOpt:
      nspsWinBlackmanOpt(@FirBuf,Length);
    wfKaiser:
      nspsWinKaiser(@FirBuf,Length,EditBeta.Value*2/(length-1));
      // nspsWinKaiser(@FirBuf,Length,0.1102*(60-8.7)*NSP_2PI/(length-1));
  end;

   ////////////////////////////////////////// NSP. FIR filtering
   ////////////////////////////////////////// NSP. FIR filtering

   /// Filter the input signals mixture to select
   /// the interesting frequency region

   // Filter initialization with array of taps and array of delay line.
   // Note that pointer to delay line array is NULL and FIR high level
   // filter calculates length of delay line and allocates memory itself.
   nspsFirInit(@FirBuf, Length, nil, State);
   // Block filtering input signal and obtaining output signal
   nspsbFir(State, @InpBuf,@OutBuf,BufLen);
   // Free allocated memory
   nspFirFree(State);

   ////////////////////////////////////////// NSP. FFT
   ////////////////////////////////////////// NSP. FFT

   // Forward FFT signal of length 2^order in place
   nspsRealFftl(@OutBuf, Order, NSP_Forw+NSP_InRCPack+NSP_OutRCPerm);
   OutBuf[0] := Sqr(OutBuf[0]);
   for i := 0 to BufLen div 2-2 do
   begin
     a := OutBuf[2*i+1];
     b := OutBuf[2*i+2];
     OutBuf[i+1] := a*a+b*b;
   end;

   /// Prepare data to draw

   // Signal spectrum

  Ymax := 0;
  Ymin := MaxSingle;
  for i := 0 to BufLen div 2-1 do
    if OutBuf[i] > 0 then
    if OutBuf[i] < Ymin then Ymin := OutBuf[i] else
    if OutBuf[i] > Ymax then Ymax := OutBuf[i];
  if Ymin > Ymax then Ymin := Ymax;
  Ysmin := Ymin;
  Ysmax := Ymax;

  SignalWin.InvalidateWorldWindow;

   // Filter coefficients

  FilterWin.WorldWidth := Length;
  FilterWin.MaxWindowWidth := Length;
  FilterWin.MinWindowWidth := Length;

  Ymin := FirBuf[0];
  Ymax := Ymin;
  for i := 1 to Length-1 do
    if FirBuf[i] < Ymin then Ymin := FirBuf[i] else
    if FirBuf[i] > Ymax then Ymax := FirBuf[i];
  Yfmin := Ymin;
  Yfmax := Ymax;

  FilterWin.InvalidateWorldWindow;
end;

procedure TForm1.TapsBarChange(Sender: TObject);
begin
  EditTaps.Value := TapsBar.Position;
end;

procedure TForm1.FpBarChange(Sender: TObject);
begin
  EditFp.Value := FpBar.Position * 0.01;
end;

procedure TForm1.FsBarChange(Sender: TObject);
begin
  EditFs.Value := FsBar.Position * 0.01;
end;

procedure TForm1.EditTapsChange(Sender: TObject);
begin
  TapsBar.Position := EditTaps.Value;
  Generate;
end;

procedure TForm1.EditFpChange(Sender: TObject);
begin
  FpBar.Position := Round(EditFp.Value * 100);
  Generate;
end;

procedure TForm1.EditFsChange(Sender: TObject);
begin
  FsBar.Position := Round(EditFs.Value * 100);
  Generate;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FilterList.ItemIndex := 0;
  WindowList.ItemIndex := wfHamming;
  SignalDisplayType.ItemIndex := 1;  
  SignalWin.WorldWidth := BufLen div 2;
  SignalWin.MaxWindowWidth := BufLen div 2;
  SignalWin.MinWindowWidth := BufLen div 2;
  Generate;
end;

procedure TForm1.SignalWinPaint(Sender: TObject);
var
  i, Y0: Integer;
  C: Extended;
  Ymin, Ymax: Single;

  function Y(X: Single): Integer;
  begin
    if X = 0
      then Result := Y0
      else Result := Round((20*log10(X)-Ymin)*C);
  end;

begin
  with SignalWin do
  begin
    if Ysmin = Ysmax then
    begin
      MoveTo(0, 0);
      LineTo(Length-1, 0);
      Exit;
    end;
    try
      if SignalDisplayType.ItemIndex = 1 then
      begin
	Y0 := WindowTop-WindowHeight+1;
	Ymin := 20*log10(Ysmin);
	Ymax := 20*log10(Ysmax);
	C := WindowTop/(Ymax-Ymin);
	MoveTo(0, Y(OutBuf[0]));
	for i := 1 to BufLen div 2-1 do
	  LineTo(i, Y(OutBuf[i]));
      end else
      begin
	C := WindowTop/Ysmax;
	MoveTo(0, Round(OutBuf[0]*C));
	for i := 1 to BufLen div 2-1 do
	  LineTo(i, Round(OutBuf[i]*C));
      end;
    except
      on EMathError do;
    end;
  end;
end;

procedure TForm1.ListChange(Sender: TObject);
begin
  EditFs.Enabled := FilterList.ItemIndex = ftBandpass;
  FsBar.Enabled := EditFs.Enabled;
  EditBeta.Enabled := WindowList.ItemIndex = wfKaiser;
  BetaBar.Enabled := EditBeta.Enabled;
  Generate;
end;

procedure TForm1.FilterWinPaint(Sender: TObject);
var
  i: Integer;
  C: Extended;
begin
  with FilterWin do
  begin
    if Yfmin = Yfmax then
    begin
      MoveTo(0, 0);
      LineTo(Length-1, 0);
      Exit;
    end;
    try
      C := WindowTop/(Yfmax-Yfmin);
      MoveTo(0, Round((FirBuf[0]-Yfmin)*C));
      for i := 1 to Length-1 do
	LineTo(i, Round((FirBuf[i]-Yfmin)*C));
    except
      on EMathError do;
    end;
  end;
end;

procedure TForm1.SignalWinWorldPosChange(Sender: TObject; X, Y: Double);
var
  Ye: Extended;
begin
  Ye := Y;
  SignalWinGetVertScalePoint(Self, Ye);
  PosX.Caption := Format('x: %f', [X]);
  PosY.Caption := Format('y: %f%*:s', [Ye, SignalDisplayType.ItemIndex+2, '', ' dB']);
end;

procedure TForm1.FilterWinWorldPosChange(Sender: TObject; X, Y: Double);
var
  Ye: Extended;
begin
  Ye := Y;
  FilterWinGetVertScalePoint(Self, Ye);
  with FilterWin do
  begin
    PosX.Caption := Format('x: %f', [X]);
    PosY.Caption := Format('y: %f', [Ye]);
  end;
end;

procedure TForm1.SignalDisplayTypeChange(Sender: TObject);
begin
  SignalWin.InvalidateWorldWindow;
end;

procedure TForm1.ShowInfoDlg(Sender: TObject);
begin
  InfoDlg.ShowModal;
end;

procedure TForm1.BetaBarChange(Sender: TObject);
begin
  EditBeta.Value := BetaBar.Position*1E-2;
end;

procedure TForm1.EditBetaChange(Sender: TObject);
begin
  BetaBar.Position := Round(EditBeta.Value*100);
  if WindowList.ItemIndex = wfKaiser then Generate;
end;

procedure TForm1.SignalWinGetVertScalePoint(Sender: TObject;
  var Value: Extended);
var
  Ymin, Ymax: Single;
begin
  with SignalWin do
  if SignalDisplayType.ItemIndex = 1 then
  begin
    Ymin := 20*log10(Ysmin);
    Ymax := 20*log10(Ysmax);
    Value := Ymin+Value*(Ymax-Ymin)/WindowTop;
  end else
  begin
    Value := Value*Ysmax/WindowTop;
  end;
end;

procedure TForm1.FilterWinGetVertScalePoint(Sender: TObject;
  var Value: Extended);
begin
  with FilterWin do Value := Yfmin+Value*(Yfmax-Yfmin)/WindowTop;
end;

procedure TForm1.SignalWinMouseLeave(Sender: TObject);
begin
  PosX.Caption := '';
  PosY.Caption := '';
end;

end.
